<header class="page-header">
  <h1><%= t('.title') %></h1>
</header>

<%= form_tag @pre_auth.redirect_uri, method: :post, name: :redirect_form, authenticity_token: false do %>
  <% auth.body.compact.each do |key, value| %>
    <%= hidden_field_tag key, value %>
  <% end %>
<% end %>

<script>
  window.onload = function () {
    document.forms['redirect_form'].submit();
  };
</script>
