# frozen_string_literal: true

class RemoveApplicationsSecretNotNullConstraint < ActiveRecord::Migration<%= migration_version %>
  def change
    change_column_null :oauth_applications, :secret, true
  end
end
